# frozen_string_literal: true

require 'influxdb_data_harvester'
require 'rspec/eventually'
require 'webmock/rspec'

describe InfluxDBDataHarvester do
  subject(:harvester) { described_class.new }

  let(:current_example) do
    metadata = {
      description: 'can send test data',
      full_description: 'InfluxdbDataHarvester can send test data',
      file_path: './spec/data_harvester_spec.rb',
      line_number: 26,
      location: './spec/data_harvester_spec.rb:26',
      absolute_file_path: '/Development/spec/data_harvester_spec.rb',
      rerun_file_path: './spec/data_harvester_spec.rb',
      last_run_status: 'passed'
    }
    current_example = RSpec.current_example
    metadata.each do |key, value|
      current_example.metadata[key] = value
    end
    current_example.execution_result.status = 'passed'
    current_example
  end

  before do
    stub_request(:any, '127.0.0.1')
    query = { db: 'master', p: 'root', precision: 'u', u: 'root' }
    body  = 'test_results duration=,result="passed",'
    body += 'name="InfluxdbDataHarvester can send test data",source_environment="local"'
    @request = a_request(:post, '127.0.0.1:8086/write').with(body: body, query: query)
  end

  it 'can send test data' do
    harvester.queue_send_data(current_example)
    # Need to find out what expectation to check here
    expect { @request }.to eventually(have_been_made.once).within 2
  end

  context 'can be customized' do
    subject(:custom_database_harvester) { described_class.new database: 'test_run_results' }

    before do
      query = { db: 'test_run_results', p: 'root', precision: 'u', u: 'root' }
      body  = 'test_results duration=,result="passed",'
      body += 'name="InfluxdbDataHarvester can send test data",source_environment="local"'
      @request = a_request(:post, '127.0.0.1:8086/write').with(body: body, query: query)
    end

    it 'to use a different database' do
      custom_database_harvester.queue_send_data(current_example)
      expect { @request }.to eventually(have_been_made.once).within 2
    end
  end

  it 'generates payload that includes full description' do
    data = harvester.generate_payload(current_example)
    expect(data[:values][:name]).to eq 'InfluxdbDataHarvester can send test data'
  end

  it 'supports custom data in the payload' do
    query = { db: 'master', p: 'root', precision: 'u', u: 'root' }
    body  = 'test_results duration=,result="passed",'
    body += 'name="InfluxdbDataHarvester can send test data",source_environment="local"'
    body += ',custom_field="custom_data"'
    request = a_request(:post, '127.0.0.1:8086/write').with(body: body, query: query)

    custom_data = {
      custom_field: 'custom_data'
    }

    harvester.queue_send_data(current_example, extra_data: custom_data)
    expect { request }.to eventually(have_been_made.once).within 2
  end
end
